﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Controller : MonoBehaviour
{
    #region public accessor
    public bool Flight { set => onflight = value; }         // takeoff T, land F

    public float Phi   { set => phi_r   = value / 5f; }     // right +, left  -
    public float Theta { set => theta_b = value / 5f; }     // back  +, front -
    public float Gaz   { set => gaz_up  = value / 5f; }     // up    +, down  -
    public float Yaw   { set => yaw_cw  = value / 5f; }     // cw    +, ccw   -

    public Vector3 Velocity => velocity;    // drone velocity navdata
    public Vector3 Rotation => rotation;    // drone rotation navdata
    public float   Altitude => altitude;    // drone altitude navdata
    #endregion

    #region private attributes
    [SerializeField] private new Rigidbody rigidbody;   // Drone rigidbody
    [SerializeField] private new Transform transform;   // Drone rotation (must be FixRotation)

    [SerializeField] private bool onflight;

    [SerializeField] private float phi_r;     
    [SerializeField] private float theta_b;   
    [SerializeField] private float gaz_up;    
    [SerializeField] private float yaw_cw;    

    [SerializeField] private Vector3 velocity;
    [SerializeField] private Vector3 rotation;
    [SerializeField] private float   altitude;
    #endregion

    #region inspector view variable
    [SerializeField] private Transform rbTransform;     // rotation of rigidbody

    [SerializeField] private Vector3 prevVelocity;
    [SerializeField] private Vector3 targetVelocity;
    [SerializeField] private Vector3 pv, iv, dv;
    [SerializeField] private Vector3 force;

    [SerializeField] private float rbRotation;
    [SerializeField] private float prevRbRotation;
    [SerializeField] private float rRVelocity;
    [SerializeField] private float prevRRVelocity;
    [SerializeField] private float targetRRVelocity;
    [SerializeField] private float prrv, irrv, drrv;
    [SerializeField] private float torque;
    #endregion

    void Start() {
        rbTransform = rigidbody.GetComponent<Transform>();
    }

    void FixedUpdate() {
        // update value
        prevVelocity = velocity;
        velocity = transform.InverseTransformDirection(rigidbody.velocity);
        rotation = transform.eulerAngles;
        altitude = transform.position.y;

        // calculate force
        if (onflight) {
            targetVelocity = new Vector3(phi_r, gaz_up, -theta_b);
            if (altitude < 1f)
                targetVelocity.y += 3 * (1f - altitude);
        }
        else {
            targetVelocity = new Vector3(0, -5f, 0);
            if (altitude < 0.55f) {
                targetVelocity.y = 0f;
                pv = iv = dv = Vector3.zero;
            }
        }

        pv = targetVelocity - velocity;
        iv = iv + pv;
        dv = velocity - prevVelocity;

        force = 30f * pv + 0.2f * iv + 50f * dv;
        force.y = 60f * pv.y + 0.4f * iv.y + 100f * dv.y;

        rigidbody.AddRelativeForce(force);

        // calculate rotation
        prevRbRotation = rbRotation;
        rbRotation = rbTransform.eulerAngles.y;
        prevRRVelocity = rRVelocity;
        rRVelocity = (rbRotation - prevRbRotation) * 60f;
        
        if (onflight) {
            targetRRVelocity = yaw_cw;
        } else {
            targetRRVelocity = 0;
            if (altitude < 0.55f)
                prrv = irrv = drrv = 0;
        }

        prrv = targetRRVelocity - rRVelocity;
        irrv = irrv + prrv;
        drrv = rRVelocity - prevRRVelocity;

        torque = 30f * prrv + 0.2f * irrv + 50f * drrv;
        
        rRVelocity += torque;
        rbRotation += rRVelocity / 60f;
        var targetRotQuat = Quaternion.Euler(0, rbRotation, 0);
        rbTransform.rotation = Quaternion.Lerp(rbTransform.rotation, targetRotQuat, 0.1f);
        
        //tgRotation = Quaternion.Euler(0, rbTransform.eulerAngles.y + yaw_cw* 10f, 0);
        //rbTransform.rotation = Quaternion.Lerp(rbRotation, tgRotation, 0.05f);

        // tilt effect
        var currentQuat = transform.localRotation;
        var targetQuat = Quaternion.Euler(-theta_b, 1, -phi_r);
        transform.localRotation = Quaternion.Lerp(currentQuat, targetQuat, 0.05f);
    }
}
