﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class ThrustController : MonoBehaviour
{
    public bool Emergency { get => !standby; set => standby = !value; }
    public float Thrust { get => thrust; set => thrust = value; }

    [SerializeField] private bool standby = true;
    [SerializeField] private float thrust;

    [SerializeField] [Readonly] private float[] rotorThrust = new float[4]; // RF, RB, LB, LF
    [SerializeField] [Readonly] private float   rotorTorque = 0f;

    private static readonly IReadOnlyList<Vector3> rotorPosition = new Vector3[4] {
        new Vector3( 0.6f, 0.25f,  0.6f),   // RF
        new Vector3( 0.6f, 0.25f, -0.6f),   // RB
        new Vector3(-0.6f, 0.25f, -0.6f),   // LB
        new Vector3(-0.6f, 0.25f,  0.6f)    // LF
    };

    private new Transform      transform;
    private new Rigidbody      rigidbody;
    private NavdataHandler     navdata;
    private AttitudeController attitude;
    
    void Start() {
        transform = GetComponent<Transform>();
        rigidbody = GetComponent<Rigidbody>();
        navdata   = GetComponent<NavdataHandler>();
        attitude  = GetComponent<AttitudeController>();
    }

    void FixedUpdate() {
        if (standby) {
            var correction = attitude.Correction;
            rotorThrust[0] = thrust - correction.x + correction.y + correction.z;
            rotorThrust[1] = thrust + correction.x - correction.y + correction.z;
            rotorThrust[2] = thrust + correction.x + correction.y - correction.z;
            rotorThrust[3] = thrust - correction.x - correction.y - correction.z;
            rotorTorque = rotorThrust[0] - rotorThrust[1] + rotorThrust[2] - rotorThrust[3];

            // apply thrust to each rotor position
            rotorPosition
                .Select(p => transform.TransformPoint(p))   // local to world position
                .Zip(rotorThrust, (p, f) => new { p, f })
                .ToList()
                .ForEach(e => rigidbody.AddForceAtPosition(transform.up * e.f, e.p));

            rigidbody.AddRelativeTorque(new Vector3(0, rotorTorque, 0));
        }
    }
}
